
#ifndef TARTANAFILE_HH
#define TARTANAFILE_HH

#include <map>
#include <vector>
#include <string>
#include <iostream>

#include "TCutG.h"
#include "TH1.h"

class TArtValIndex
{
public:  
  TArtValIndex(){;}
  TArtValIndex(int analyser, int wnum, int index)
    : fAnalyser(analyser), fWNum(wnum), fIndex(index){;}
  virtual ~TArtValIndex(){;}

  friend std::ostream& operator <<
  (std::ostream& out, const TArtValIndex& right)
  {
    return out << right.fAnalyser << " " << right.fWNum << " " << right.fIndex;
  }

  bool operator < (const TArtValIndex& right) const
  {
    if(fAnalyser != right.fAnalyser){
      return fAnalyser < right.fAnalyser;
    }else if(fWNum != right.fWNum){
      return fWNum < right.fWNum;
    }else{
      return fIndex < right.fIndex;
    }
  }

  int fAnalyser;
  int fWNum;
  int fIndex;
 
  ClassDef(TArtValIndex,1);
};

class TArtAnaGate
{
public:
  TArtAnaGate()
    : fGate(0), fInside(true), fUL(false), fLow(0), fUp(-1),
      fAnd(false), fOr(false),
      fCutG(0), fCC(false){;}
  TArtAnaGate(int gate, double low, double up)
    : fGate(gate), fInside(false), fUL(true), fLow(low), fUp(up),
      fAnd(false), fOr(false),
      fCutG(0), fCC(false){;}
  TArtAnaGate(int gate, bool And, bool Or, const std::vector<int>& gatearray)
    : fGate(gate), fInside(false), fUL(false), fLow(0), fUp(-1),
      fAnd(And), fOr(Or), fGateArray(gatearray),
      fCutG(0), fCC(false){;}
  TArtAnaGate(int gate, TCutG* cutg)
    : fGate(gate), fInside(false), fUL(false), fLow(0), fUp(-1),
      fAnd(false), fOr(false),
      fCutG(cutg), fCC(false){;}
  TArtAnaGate(int gate, double centerx, double centery, double radiusx, double radiusy)
    : fGate(gate), fInside(false), fUL(false), fLow(0), fUp(-1),
      fAnd(false), fOr(false),
      fCutG(0), fCC(true),
      fCenterX(centerx), fCenterY(centery), fRadiusX(radiusx), fRadiusY(radiusy){;}
  virtual ~TArtAnaGate(){;}

  int fGate;
  bool fInside;

  bool fUL;
  double fLow, fUp;

  bool fAnd, fOr;
  std::vector<int> fGateArray;

  TCutG* fCutG;

  bool fCC;
  double fCenterX, fCenterY, fRadiusX, fRadiusY;

  ClassDef(TArtAnaGate,1);
};

class TArtAnaFile
{
#ifndef __CINT__
  //http://root.cern.ch/phpBB3/viewtopic.php?f=5&t=10451
  typedef bool (TArtAnaFile::*FSequence)(std::string, int);
#endif

public:
  TArtAnaFile(){
    fHistNameCounter = 0;
  }
  virtual ~TArtAnaFile(){;}

  bool Construct();
  bool Open(const char* filename);
  bool Check();
  void ClearVal();
  void ClearMap();
  bool Add(int analyser, int wnum, int index, Double_t val);
  void Fill();

  void ChangeAnaFile(std::string anafilename);

  TCutG* CreateCutG(int id = 0, const char* filename = 0, const char* cutname = 0);

private:
  bool CalGate(int gate);
  bool IsInside(int gate);
  bool Find(TArtValIndex valindex, Double_t& val);

  bool AddValIndexMap(std::string name, std::vector<TArtValIndex> valindexarray);
  bool AddGateMap(std::string name, int gate);
  bool AddValIndexArrayMap(int gate, std::vector<TArtValIndex> valindexarray);
  bool AddAnaGateMap(int gate, TArtAnaGate anagate);
  bool AddSequenceMap(std::string name, FSequence fp);

  std::map<TArtValIndex, Double_t> fValMap;

  std::vector<TH1*> fH1Array;
  std::map<std::string, std::vector<TArtValIndex> > fValIndexMap;
  std::map<std::string, int> fGateMap;

  std::map<int, TArtAnaGate> fAnaGateMap;
  std::map<int, std::vector<TArtValIndex> > fValIndexArrayMap;

  std::map<int, TCutG*> fCutGMap;

  int fHistNameCounter;

private:
  bool Hist(std::string str, int countline);
  bool And(std::string str, int countline);
  bool Or(std::string str, int countline);
  bool Gate(std::string str, int countline);
  bool CC(std::string str, int countline);
  bool CutG(std::string str, int countline);
  bool Point(std::string str, int countline);
  bool Include(std::string str, int countline);

  bool ReadSequence(std::string str, int countline, std::string& sequence);
  bool ReadValues(std::string str, int countline, std::vector<double>& x);
  bool ReadValues2(std::string str, int countline, std::vector<double>& x);
  bool ReadName(std::string str, int countline, std::string& name);
  bool ReadTitle(std::string str, int countline, std::string& title);

#ifndef __CINT__
  std::map<std::string, FSequence> fSequenceMap;
#endif

  ClassDef(TArtAnaFile,1);
};

#endif
